/************************************************************************

   SERVO Controller for up to 10 Servos

   controlled by serial line
   default: 2400 Baud 8N1

   Processor: ATMEGA 8
   CLOCK: 8MHZ, no prescaler set config bits of Atmega 8 correctly !

   Compiler: AVR-GCC


   This code is licensed under the GPL.
   You may modify, redistribute the code .. blabla, you know what I mean ...

   Copyright stochri (c.hab@gmx.net) Nov.2005

   -----
   Modified by Andi Kanzler (andi_kanzler@gmx.de), Mar. 2010

***************************************************************************/

#define TIMER_MAXPULS SYSCLK/MAXPULSFREQ // Timer1 value for a 2ms Puls
#define MAXPULSFREQ 500 // 2ms => 500HZ

#define MINPULS TIMER_MAXPULS/4  // min pulslength = 0.5ms
#define MAXPULS TIMER_MAXPULS // max pulslength=2ms



#include "servo.h"

uint16_t Pulslength[20]; // array for all delays

/************************************************************************

   SIGNAL(SIG_OVERFLOW1)
   timer1 interrupt, generates the high and low pulses for each servo

***************************************************************************/
SIGNAL(SIG_OVERFLOW1)
{
   static uint8_t servoindex_half=0;

   switch (servoindex_half)
   {
      case 0: HIGH_SERVO0; break;
      case 1: LOW_SERVO0; break;
      case 2: HIGH_SERVO1; break;
      case 3: LOW_SERVO1; break;
      case 4: HIGH_SERVO2; break;
      case 5: LOW_SERVO2; break;
      case 6: HIGH_SERVO3; break;
      case 7: LOW_SERVO3; break;
      case 8: HIGH_SERVO4; break;
      case 9: LOW_SERVO4; break;
      case 10: HIGH_SERVO5; break;
      case 11: LOW_SERVO5; break;
      case 12: HIGH_SERVO6; break;
      case 13: LOW_SERVO6; break;
      case 14: HIGH_SERVO7; break;
      case 15: LOW_SERVO7; break;
      case 16: HIGH_SERVO8; break;
      case 17: LOW_SERVO8; break;
      case 18: HIGH_SERVO9; break;
      case 19: LOW_SERVO9; break;
   }


   TCNT1 =Pulslength[servoindex_half]; // set time for next interrupt

    servoindex_half++; // increment timervalue index
   if(servoindex_half==20)servoindex_half=0;   // reset index
}
/************************************************************************

   void setservo(byte index, byte value)

   Set servo position
   value: 0..255

***************************************************************************/

void setservo(uint8_t index, uint8_t value)
{
   // Prevent Servo from mechanical damage
   if (value < MINVALUE)
       value = MINVALUE;

    if (value > MAXVALUE)
        value = MAXVALUE;

   uint16_t wert;

   wert=MINPULS+(MAXPULS-MINPULS)/256*value;

   // callculate hightime
   Pulslength[index<<1]=0-wert;

   // sume of low and hightime for one servo is 2ms
   Pulslength[(index<<1)+1]=0-(TIMER_MAXPULS-wert);

   // 10 servos give you 10*2ms=20ms total cycle time
}

/************************************************************************

   void init_servos()

   initialize all servos to the start position
***************************************************************************/
void init_servos()
{

// init timer1
   TCNT1 = 0-16000;
   TCCR1A=0;
   TCCR1B=0x01;
   TIMSK |= _BV(TOIE2) | _BV(TOIE1);

    /* allow interrupts */
    sei();

   // Set the Startvalues
   uint8_t values[] = STARTVALUES;
   uint8_t i;
   for(i = 0; i < 10; i++)
       setservo(i, values[i]);
}
